/*******************************************************************************
 * Copyright (c) 2008 Mateusz Matela and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     Mateusz Matela - initial API and implementation
 *******************************************************************************/
package org.gsoc.eclipse.tostringgenerator;

import java.util.List;

import org.eclipse.core.runtime.CoreException;

import org.eclipse.ltk.core.refactoring.RefactoringStatus;

import org.eclipse.jdt.core.dom.MethodDeclaration;

/**
 * The interface for generator of toString() method that is used when the
 * 'Generate toString()' action is invoked.
 * 
 * This interface can be implemented directly, but it is recommended to extend
 * {@link org.eclipse.jdt.ui.tostringgeneration.AbstractToStringGenerator}
 * instead as it provides a lot of common functionality and convenience methods.
 */
public interface IToStringGenerator {

	/**
	 * The name of the property that every <code>MethodDeclaration</code>
	 * generated should have. This property determines whether the method should
	 * be overwritten if already exists. The data of this property is a Boolean.
	 */
	public final String OVERWRITE_METHOD_PROPERTY = "override_method"; //$NON-NLS-1$

	/**
	 * 
	 * @return the parser to be used with this generator
	 */
	public IToStringTemplateParser getTemplateParser();

	/**
	 * 
	 * @return a refactoring status with eventual errors and warnings caused by
	 *         current context
	 */
	public RefactoringStatus checkConditions();

	/**
	 * Runs the generator using current context
	 * 
	 * @return the generated toString() method
	 * @throws CoreException
	 */
	public MethodDeclaration generateToStringMethod() throws CoreException;

	/**
	 * Generates and returns helper methods, if necessary. This method is called
	 * after {@link #generateToStringMethod()} and should use the same context
	 * information.
	 * 
	 * @return the list of <code>MethodDeclaration</code> for helper methods
	 *         (empty list if no helper method is needed)
	 */
	public List generateHelperMethods();

	/**
	 * 
	 * @return the current context
	 */
	public IToStringGenerationContext getContext();

	/**
	 * sets the current context
	 * 
	 * @param context
	 *            context to be set
	 */
	public void setContext(IToStringGenerationContext context);

}
